Package: rsem
Type: Package
Title: Robust Structural Equation Modeling with Missing Data and
        Auxiliary Variables
Version: 0.5.1
Date: 2023-08-26
Author: Ke-Hai Yuan and Zhiyong Zhang
Maintainer: Zhiyong Zhang <zhiyongzhang@nd.edu>
Depends: R (>= 2.7), MASS, lavaan
Description: A robust procedure is implemented to estimate means and covariance matrix of multiple variables with missing data using Huber weight and then to estimate a structural equation model.
License: GPL-2
URL: https://bigdatalab.nd.edu
ZipData: no
LazyLoad: yes
NeedsCompilation: no
Repository: CRAN
Packaged: 2023-08-26 20:08:52 UTC; zzhang4
Date/Publication: 2023-08-26 20:40:02 UTC
Built: R 4.6.0; ; 2026-04-27 07:34:45 UTC; unix
